@@ -7,6 +7,9 @@ use Tangram::Ref;
 use Tangram::Schema;
 use Tangram::Cursor;
 use Tangram::Storage;
+use Tangram::Dialect;
 use Tangram::Expr;
 
+$Tangram::TRACE = \*STDERR if exists $ENV{TANGRAM_TRACE};
+
 1;
