@@ -3,6 +3,20 @@ use strict;
 package Tangram::AbstractStorage;
 
 use Carp;
+
+BEGIN {
+
+  eval { require 'WeakRef.pm' };
+
+  if ($@) {
+    *Tangram::weaken = sub { };
+    $Tangram::no_weakrefs = 1;
+  } else {
+    *Tangram::weaken = \&WeakRef::weaken;
+    $Tangram::no_weakrefs = 0;
+  }
+}
+
 use vars qw( %done );
 
 sub new
@@ -17,12 +31,12 @@ sub schema
 }
 
 sub _open
-{   
+  {
     my ($self, $schema) = @_;
 
     $self->{table_top} = 0;
     $self->{free_tables} = [];
-   
+
     $self->{tx} = [];
 
     $self->{schema} = $schema;
@@ -30,16 +44,15 @@ sub _open
     my $cursor = $self->sql_cursor("SELECT classId, className FROM $schema->{class_table}", $self->{db});
 
     my $classes = $schema->{classes};
-   
+
     my $id2class = {};
     my $class2id = {};
 
     my ($classId, $className);
 
-    while (($classId, $className) = $cursor->fetchrow())
-    {
-		$id2class->{$classId} = $className;
-		$class2id->{$className} = $classId;
+    while (($classId, $className) = $cursor->fetchrow()) {
+      $id2class->{$classId} = $className;
+      $class2id->{$className} = $classId;
     }
 
     $cursor->close();
@@ -47,32 +60,28 @@ sub _open
     $self->{id2class} = $id2class;
     $self->{class2id} = $class2id;
 
-    foreach my $class (keys %$classes)
-    {
-		warn "no class id for '$class'\n"
-			if $classes->{$class}{concrete} && !exists $self->{class2id}{$class};
+    foreach my $class (keys %$classes) {
+      warn "no class id for '$class'\n"
+	if $classes->{$class}{concrete} && !exists $self->{class2id}{$class};
     }
 
     $self->{set_id} = $schema->{set_id} ||
-    	sub
-		{
-			my ($obj, $id) = @_;
-         
-			if ($id)
-			{
-				$self->{ids}{0 + $obj} = $id;
-			}
-			else
-			{
-				delete $self->{ids}{0 + $obj};
-			}
-		};
+      sub
+	{
+	  my ($obj, $id) = @_;
+
+	  if ($id) {
+	    $self->{ids}{0 + $obj} = $id;
+	  } else {
+	    delete $self->{ids}{0 + $obj};
+	  }
+	};
 
     $self->{get_id} = $schema->{get_id}
-		|| sub { $self->{ids}{0 + shift()} };
+      || sub { $self->{ids}{0 + shift()} };
 
     return $self;
-}
+  }
 
 sub alloc_table
 {
@@ -148,28 +157,34 @@ sub my_select_data
 }
 
 sub make_id
-{
+  {
     my ($self, $class_id) = @_;
+    return $self->{dialect}->make_id($self, $class_id);
+  }
 
-	my $alloc_id = $self->{alloc_id} ||= {};
-
-	my $id = $alloc_id->{$class_id};
-
-	if ($id)
-	{
-		$id = -$id if $id < 0;
-		$alloc_id->{$class_id} = ++$id;
-	}
-	else
-	{
-		my $table = $self->{schema}{class_table};
-		$self->sql_do("UPDATE $table SET lastObjectId = lastObjectId + 1 WHERE classId = $class_id");
-		$id = $self->sql_selectall_arrayref(
-	        "SELECT lastObjectId from $table WHERE classId = $class_id")->[0][0];
-		$alloc_id->{$class_id} = -$id;
-	}
-
-	return sprintf "%d%0$self->{cid_size}d", $id, $class_id;
+sub std_make_id
+  {
+    my ($self, $class_id) = @_;
+    
+    my $alloc_id = $self->{alloc_id} ||= {};
+    
+    my $id = $alloc_id->{$class_id};
+    
+    if ($id)
+      {
+	$id = -$id if $id < 0;
+	$alloc_id->{$class_id} = ++$id;
+      }
+    else
+      {
+	my $table = $self->{schema}{class_table};
+	$self->sql_do("UPDATE $table SET lastObjectId = lastObjectId + 1 WHERE classId = $class_id");
+	$id = $self->sql_selectall_arrayref(
+					    "SELECT lastObjectId from $table WHERE classId = $class_id")->[0][0];
+	$alloc_id->{$class_id} = -$id;
+      }
+    
+    return sprintf "%d%0$self->{cid_size}d", $id, $class_id;
 }
 
 sub unknown_classid
@@ -190,66 +205,84 @@ sub class_id
 my $error_no_transaction = 'no transaction is currently active';
 
 sub tx_start
+{
+  my $self = shift;
+  $self->{dialect}->tx_start($self);
+}
+
+sub std_tx_start
 {
     my $self = shift;
     push @{ $self->{tx} }, [];
 }
 
 sub tx_commit
-{
+  {
     # public - commit current transaction
-
     my $self = shift;
+    $self->{dialect}->tx_commit($self);
+  }
 
+sub std_tx_commit
+  {
+    # public - commit current transaction
+    
+    my $self = shift;
+    
     carp $error_no_transaction unless @{ $self->{tx} };
-
-	# update lastObjectId's
-
-	if (my $alloc_id = $self->{alloc_id})
-	{
-		my $table = $self->{schema}{class_table};
-
-		for my $class_id (keys %$alloc_id)
-		{
-			my $id = $alloc_id->{$class_id};
-			next if $id < 0;
-			$self->sql_do("UPDATE $table SET lastObjectId = $id WHERE classId = $class_id");
-		}
-
-		delete $self->{alloc_id};
-	}
-
-	unless ($self->{no_tx} || @{ $self->{tx} } > 1)
-	{
-		# committing outer tx: commit to db
-		$self->{db}->commit;
-	}
+    
+    # update lastObjectId's
+    
+    if (my $alloc_id = $self->{alloc_id})
+      {
+	my $table = $self->{schema}{class_table};
+	
+	for my $class_id (keys %$alloc_id)
+	  {
+	    my $id = $alloc_id->{$class_id};
+	    next if $id < 0;
+	    $self->sql_do("UPDATE $table SET lastObjectId = $id WHERE classId = $class_id");
+	  }
+	
+	delete $self->{alloc_id};
+      }
+    
+    unless ($self->{no_tx} || @{ $self->{tx} } > 1)
+      {
+	# committing outer tx: commit to db
+	$self->{db}->commit;
+      }
 
     pop @{ $self->{tx} };		# drop rollback subs
 }
 
 sub tx_rollback
-{
+  {
     # public - rollback current transaction
-
     my $self = shift;
-
+    $self->{dialect}->tx_rollback($self);
+  }
+    
+sub std_tx_rollback
+  {
+    my $self = shift;
+    
     carp $error_no_transaction unless @{ $self->{tx} };
-
+    
     if ($self->{no_tx})
-    {
-		pop @{ $self->{tx} };
-    }
+      {
+	pop @{ $self->{tx} };
+      }
     else
-    {
-		$self->{db}->rollback if @{ $self->{tx} } == 1; # don't rollback db if nested tx
-
-		# execute rollback subs in reverse order
-
-		foreach my $rollback ( @{ pop @{ $self->{tx} } } )
-		{
-			$rollback->($self);
-		}
+      {
+	$self->{db}->rollback if @{ $self->{tx} } == 1; # don't rollback db if nested tx
+	
+	# execute rollback subs in reverse order
+	
+	foreach my $rollback ( @{ pop @{ $self->{tx} } } )
+	  {
+	    $rollback->($self);
+	  }
     }
 }
 
@@ -329,7 +362,8 @@ sub _insert
     my ($self, $obj) = @_;
     my $schema = $self->{schema};
 
-    return $self->id($obj) if $self->id($obj);
+    return $self->id($obj)
+      if $self->id($obj);
 
     $done{$obj} = 1;
 
@@ -338,15 +372,14 @@ sub _insert
 
     my $id = $self->make_id($classId);
 
-    $self->{objects}{$id} = $obj;
-    $self->{set_id}->($obj, $id);
-    $self->tx_on_rollback( sub { $self->{set_id}->($obj, undef) } );
+    $self->welcome($obj, $id);
+    $self->tx_on_rollback( sub { $self->goodbye($obj, $id) } );
 
     $schema->visit_up($class,
 	    sub
 		{
 			my ($class) = @_;
-         
+
 			my $classdef = $schema->classdef($class);
 
 			my $table = $classdef->{table};
@@ -382,7 +415,7 @@ sub auto_insert
 {
     # private - convenience sub for Refs, will be moved there someday
 
-    my ($self, $obj, $table, $col, $id) = @_;
+    my ($self, $obj, $table, $col, $id, $deep_update) = @_;
 
     return 'NULL' unless $obj;
 
@@ -402,8 +435,11 @@ sub auto_insert
 	return 'NULL';
     }
 
-    return $self->id($obj)	# already persistent
-	|| $self->_insert($obj); # autosave
+    $self->_save($obj) if $deep_update;
+
+    return $self->id($obj)     # already persistent
+      || $self->_insert($obj); # autosave
+
 }
 
 #############################################################################
@@ -421,46 +457,54 @@ sub update
 		     my ($self, @objs) = @_;
 		     foreach my $obj (@objs)
 		     {
-			 my $id = $self->id($obj) or confess "$obj must be persistent";
-   
 			 local %done = ();
 			 local $self->{defered} = [];
 
-			 my $class = ref $obj;
-			 my $schema = $self->{schema};
-			 my $types = $schema->{types};
-
-			 $schema->visit_up($class,
-					   sub
-					   {
-					       my ($class) = @_;
-
-					       my $classdef = $schema->classdef($class);
-
-					       my $table = $classdef->{table};
-					       my @cols = ();
-					       my @vals = ();
-
-					       foreach my $typetag (keys %{$classdef->{members}})
-					       {
-						   $types->{$typetag}->save(\@cols, \@vals, $obj,
-									    $classdef->{members}{$typetag},
-									    $self, $table, $id);
-					       }
-
-					       if (@cols)
-					       {
-						   my $assigns = join ', ', map { "$_ = " . shift @vals } @cols;
-						   my $update = "UPDATE $table SET $assigns WHERE id = $id";
-						   $self->sql_do($update);
-					       }
-					   } );
-
+			 $self->_update($obj);
 			 $self->do_defered;
-		       }
+		     }
 		   }, $self, @objs);
 }
 
+sub _update
+{
+    my ($self, $obj) = @_;
+
+    my $id = $self->id($obj) or confess "$obj must be persistent";
+   
+    $done{$obj} = 1;
+    
+    my $class = ref $obj;
+    my $schema = $self->{schema};
+    my $types = $schema->{types};
+
+    $schema->visit_up($class,
+		      sub
+		      {
+			my ($class) = @_;
+			
+			my $classdef = $schema->classdef($class);
+			
+			my $table = $classdef->{table};
+			my @cols = ();
+			my @vals = ();
+			
+			foreach my $typetag (keys %{$classdef->{members}})
+			  {
+			    $types->{$typetag}->save(\@cols, \@vals, $obj,
+						     $classdef->{members}{$typetag},
+						     $self, $table, $id);
+			  }
+			
+			if (@cols)
+			  {
+			    my $assigns = join ', ', map { "$_ = " . shift @vals } @cols;
+			    my $update = "UPDATE $table SET $assigns WHERE id = $id";
+			    $self->sql_do($update);
+			  }
+		      } );
+}
+
 #############################################################################
 # save
 
@@ -481,63 +525,72 @@ sub save
     }
 }
 
+sub _save
+{
+    my $self = shift;
+    foreach my $obj (@_)
+    {
+        if ($self->id($obj))
+	{
+	    $self->_update($obj)
+	}
+	else
+	{
+	    $self->_insert($obj)
+	}
+    }
+}
+
+
 #############################################################################
 # erase
 
 sub erase
-{
+  {
     my ($self, @objs) = @_;
 
-	$self->tx_do(
-        sub
-		{
-			my ($self, @objs) = @_;
-			my $schema = $self->{schema};
-			my $classes = $self->{schema}{classes};
+    $self->tx_do(
+		 sub
+		 {
+		   my ($self, @objs) = @_;
+		   my $schema = $self->{schema};
+		   my $classes = $self->{schema}{classes};
 
-			foreach my $obj (@objs) # causes memory leak??
-			{
-				my $id = $self->id($obj) or confess "object $obj is not persistent";
+		   foreach my $obj (@objs) # causes memory leak??
+		     {
+		       my $id = $self->id($obj) or confess "object $obj is not persistent";
 
-				local $self->{defered} = [];
+		       local $self->{defered} = [];
       
-				$schema->visit_down(ref($obj),
-				    sub
-					{
-						my $class = shift;
-						my $classdef = $classes->{$class};
+		       $schema->visit_down(ref($obj),
+					   sub
+					   {
+					     my $class = shift;
+					     my $classdef = $classes->{$class};
 
-						foreach my $typetag (keys %{$classdef->{members}})
-						{
-							my $members = $classdef->{members}{$typetag};
-							my $type = $schema->{types}{$typetag};
-							$type->erase($self, $obj, $members, $id);
-						}
-					} );
-      
-				$schema->visit_down(ref($obj),
-					sub
-					{
-						my $class = shift;
-						my $classdef = $classes->{$class};
-						$self->sql_do("DELETE FROM $classdef->{table} WHERE id = $id")
-							unless $classdef->{stateless};
-					} );
+					     foreach my $typetag (keys %{$classdef->{members}}) {
+					       my $members = $classdef->{members}{$typetag};
+					       my $type = $schema->{types}{$typetag};
+					       $type->erase($self, $obj, $members, $id);
+					     }
+					   } );
 
-				$self->do_defered;
+		       $schema->visit_down(ref($obj),
+					   sub
+					   {
+					     my $class = shift;
+					     my $classdef = $classes->{$class};
+					     $self->sql_do("DELETE FROM $classdef->{table} WHERE id = $id")
+					       unless $classdef->{stateless};
+					   } );
 
-				delete $self->{objects}{$id};
-				$self->{set_id}->($obj, undef);
+		       $self->do_defered;
 
-				$self->tx_on_rollback(
-				    sub
-					{
-						$self->{objects}{$id} = $obj;
-						$self->{set_id}->($obj, $id);
-					} );
-			}
-		}, $self, @objs );
-}
+		       $self->goodbye($obj, $id);
+		       $self->tx_on_rollback( sub { $self->welcome($obj, $id) } );
+		     }
+		 }, $self, @objs );
+  }
 
 sub do_defered
 {
@@ -566,16 +619,17 @@ sub load
     my $id = shift;
     die if @_;
 
-    return $self->{objects}{$id} if exists $self->{objects}{$id};
+    return $self->{objects}{$id}
+      if exists $self->{objects}{$id} && defined $self->{objects}{$id};
 
     my $class = $self->{id2class}{ int(substr($id, -$self->{cid_size})) };
 
 	my ($row, $alias) = _fetch_object_state($self, $id, $class);
 
     my $obj = $self->read_object($id, $class, $row, $alias->parts);
-   
+
     # ??? $self->{-residue} = \@row;
-   
+
     return $obj;
 }
 
@@ -591,30 +645,58 @@ sub reload
 
 	my ($row, $alias) = _fetch_object_state($self, $id, $class);
     _row_to_object($self, $obj, $id, $class, $row, $alias->parts);
-   
+
     return $obj;
 }
 
+sub welcome
+  {
+    my ($self, $obj, $id) = @_;
+    $self->{set_id}->($obj, $id);
+    Tangram::weaken( $self->{objects}{$id} = $obj );
+  }
+
+sub goodbye
+  {
+    my ($self, $obj, $id) = @_;
+    $self->{set_id}->($obj, undef) if $obj;
+    delete $self->{objects}{$id};
+    delete $self->{PREFETCH}{$id};
+  }
+
+sub shrink
+  {
+    my ($self) = @_;
+
+    my $objects = $self->{objects};
+    my $prefetch = $self->{prefetch};
+
+    for my $id (keys %$objects)
+      {
+	next if $objects->{$id};
+	delete $objects->{$id};
+	delete $prefetch->{$id};
+      }
+  }
+
 sub read_object
-{
+  {
     my ($self, $id, $class, $row, @parts) = @_;
 
     my $schema = $self->{schema};
 
     my $obj = $schema->{make_object}->($class);
 
-    unless (exists	$self->{objects}{$id})
-    {
-		# do this only if object is not loaded yet
-		# otherwise we're just skipping columns in $row
-		$self->{set_id}->($obj, $id);
-		$self->{objects}{$id} = $obj;
+    unless (exists $self->{objects}{$id} && defined $self->{objects}{$id}) {
+      # do this only if object is not loaded yet
+      # otherwise we're just skipping columns in $row
+      $self->welcome($obj, $id);
     }
 
     _row_to_object($self, $obj, $id, $class, $row, @parts);
 
     return $obj;
-}
+  }
 
 sub _row_to_object
 {
@@ -703,9 +785,34 @@ sub _fetch_object_state
 sub select
 {
     croak "valid only in list context" unless wantarray;
-    my ($self, $class, @args) = @_;
-    my $cursor = Tangram::Cursor->new($self, $class, $self->{db});
-    $cursor->select(@args);
+
+    my ($self, $target, @args) = @_;
+
+    if (ref($target) eq 'ARRAY')
+      {
+	my ($first, @others) = @$target;
+
+	my @cache = map { $self->select( $_, @args ) } @others;
+
+	my $cursor = Tangram::Cursor->new($self, $first, $self->{db});
+	$cursor->retrieve( map { $_->{id} } @others );
+	
+	my $obj = $cursor->select( @args );
+	my @results;
+
+	while ($obj)
+	  {
+	    push @results, [ $obj, map { $self->load($_) } $cursor->residue() ];
+	    $obj = $cursor->next();
+	  }
+
+	return @results;
+      }
+    else
+      {
+	my $cursor = Tangram::Cursor->new($self, $target, $self->{db});
+	$cursor->select(@args);
+      }
 }
 
 sub cursor_object
@@ -771,6 +878,29 @@ sub count
     return ($self->{db}->selectrow_array($sql))[0];
 }
 
+sub sum
+{
+    my ($self, $expr, $filter) = @_;
+
+    my $objects = Set::Object->new($expr->objects);
+
+    my @filter_expr;
+
+    if ($filter)
+    {
+	$objects->insert($filter->objects);
+	@filter_expr = ( "($filter->{expr})" );
+    }
+
+    my $sql = "SELECT SUM($expr->{expr}) FROM " . join(', ', map { $_->from } $objects->members);
+
+    $sql .= "\nWHERE " . join(' AND ', @filter_expr, map { $_->where } $objects->members);
+
+    print $Tangram::TRACE "$sql\n" if $Tangram::TRACE;
+
+    return ($self->{db}->selectrow_array($sql))[0];
+}
+
 sub id
 {
     my ($self, $obj) = @_;
@@ -881,10 +1011,15 @@ sub connect
 
     @$self{ -cs, -user, -pw } = ($cs, $user, $pw);
 
-	$self->{cid_size} = $schema->{sql}{cid_size};
+    $self->{cid_size} = $schema->{sql}{cid_size};
 
-	my $dialect = $opts->{dialect} || 'Tangram::Dialect';
-	$self->{dialect} = ref($dialect) ? $dialect : $dialect->new();
+    my $dialect = $opts->{dialect} || Tangram::Dialect->guess($cs)
+      || 'Tangram::Dialect';
+
+    $dialect = $self->{dialect} = ref($dialect) ? $dialect : $dialect->new();
+
+    print $Tangram::TRACE "using dialect ", ref($dialect), "\n"
+       if $Tangram::TRACE;
 	
     $self->_open($schema);
 
@@ -929,50 +1064,24 @@ sub sql_cursor
 }
 
 sub unload
-{
-	my $self = shift;
+  {
+    my $self = shift;
+    my $objects = $self->{objects};
 
-	my $objects = $self->{objects};
-	my $set_id = $self->{set_id};
-
-	if (@_)
-	{
-		my $prefetch = $self->{PREFETCH};
-
-		for my $arg (@_)
-		{
-			my $id;
-
-			if (ref $arg)
-			{
-				$id = $self->id($arg);
-				$set_id->($arg, undef);
-			}
-			else
-			{
-				$id = $arg;
-				$set_id->($objects->{$arg}, undef);
-			}
-
-			delete $objects->{$id};
-			delete $prefetch->{$id};
-		}
+    if (@_) {
+      for my $item (@_) {
+	if (ref $item) {
+	  $self->goodbye($item, $self->{get_id}->($item));
+	} else {
+	  $self->goodbye($objects->{$item}, $item);
 	}
-	else
-	{
-		for my $obj (keys %$objects)
-		{
-			$set_id->($obj, undef);
-			delete $self->{objects}
-		}
-
-		delete $self->{objects};
-		delete $self->{ids};
-		delete $self->{PREFETCH};
-	}
-
-	undef($objects);
-}
+      }
+    } else {
+      for my $id (keys %$objects) {
+	$self->goodbye($objects->{$id}, $id);
+      }
+    }
+  }
 
 *reset = \&unload; # deprecated, use unload() instead
 
